@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2012-2018 Kohei Yoshida
+ * Copyright (c) 2012-2021 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -35,7 +35,6 @@
 #endif
 
 #include "multi_type_vector.hpp"
-#include "multi_type_vector_trait.hpp"
 
 namespace mdds {
 
@@ -46,9 +45,9 @@ namespace mtm {
  */
 enum element_t
 {
-    element_empty   = mdds::mtv::element_type_empty,
+    element_empty = mdds::mtv::element_type_empty,
     element_boolean = mdds::mtv::element_type_boolean,
-    element_string  = mdds::mtv::element_type_string,
+    element_string = mdds::mtv::element_type_string,
     element_numeric = mdds::mtv::element_type_double,
     element_integer = mdds::mtv::element_type_int32
 };
@@ -56,39 +55,45 @@ enum element_t
 /**
  * Default matrix trait that uses std::string as its string type.
  */
-struct std_string_trait
+struct std_string_traits
 {
     typedef mdds::mtv::int32_element_block integer_element_block;
     typedef mdds::mtv::string_element_block string_element_block;
-
-    typedef mdds::mtv::element_block_func element_block_func;
 };
 
-}
+} // namespace mtm
 
 /**
  * Matrix that can store numeric, integer, boolean, empty and string types.
  * The string and integer types can be specified in the matrix trait
  * template parameter. To use std::string as the string type and int as the
- * integer type, use mdds::mtm::std_string_trait.
+ * integer type, use mdds::mtm::std_string_traits.
  *
  * Internally it uses mdds::multi_type_vector as its value store.  The
  * element values are linearly stored in column-major order.
  */
-template<typename _MtxTrait>
+template<typename Traits>
 class multi_type_matrix
 {
-    typedef _MtxTrait matrix_trait;
+    typedef Traits traits_type;
+
 public:
-    typedef typename matrix_trait::string_element_block string_block_type;
-    typedef typename matrix_trait::integer_element_block integer_block_type;
+    typedef typename traits_type::string_element_block string_block_type;
+    typedef typename traits_type::integer_element_block integer_block_type;
 
     typedef typename string_block_type::value_type string_type;
     typedef typename integer_block_type::value_type integer_type;
-    typedef size_t      size_type;
+    typedef size_t size_type;
 
 private:
-    typedef mdds::multi_type_vector<typename matrix_trait::element_block_func> store_type;
+    struct mtv_trait : public mdds::mtv::default_traits
+    {
+        using block_funcs = mdds::mtv::element_block_funcs<
+            mdds::mtv::boolean_element_block, mdds::mtv::int8_element_block, mdds::mtv::double_element_block,
+            typename traits_type::string_element_block, typename traits_type::integer_element_block>;
+    };
+
+    using store_type = mdds::multi_type_vector<mtv_trait>;
 
 public:
     typedef typename store_type::position_type position_type;
@@ -103,22 +108,30 @@ public:
     {
         size_type row;
         size_type column;
-        size_pair_type() : row(0), column(0) {}
-        size_pair_type(size_type _row, size_type _column) : row(_row), column(_column) {}
+        size_pair_type() : row(0), column(0)
+        {}
+        size_pair_type(size_type _row, size_type _column) : row(_row), column(_column)
+        {}
         size_pair_type(std::initializer_list<size_type> vs)
         {
             if (vs.size() != 2)
                 throw invalid_arg_error("size_pair_type requires exactly 2 elements.");
 
-            size_type* ptrs[2] = { &row, &column };
+            size_type* ptrs[2] = {&row, &column};
             size_type** p = ptrs;
 
             for (size_type v : vs)
                 **p++ = v;
         }
 
-        bool operator== (const size_pair_type& r) const { return row == r.row && column == r.column; }
-        bool operator!= (const size_pair_type& r) const { return !operator== (r); }
+        bool operator==(const size_pair_type& r) const
+        {
+            return row == r.row && column == r.column;
+        }
+        bool operator!=(const size_pair_type& r) const
+        {
+            return !operator==(r);
+        }
     };
 
     struct element_block_node_type
@@ -162,13 +175,14 @@ public:
     }
 
 private:
-    template<typename _Func>
+    template<typename FuncT>
     struct walk_func
     {
-        _Func& m_func;
-        walk_func(_Func& func) : m_func(func) {}
+        FuncT& m_func;
+        walk_func(FuncT& func) : m_func(func)
+        {}
 
-        void operator() (const typename store_type::const_iterator::value_type& mtv_node)
+        void operator()(const typename store_type::const_iterator::value_type& mtv_node)
         {
             element_block_node_type mtm_node;
             mtm_node.type = to_mtm_type(mtv_node.type);
@@ -250,10 +264,10 @@ public:
      */
     ~multi_type_matrix();
 
-    bool operator== (const multi_type_matrix& other) const;
-    bool operator!= (const multi_type_matrix& other) const;
+    bool operator==(const multi_type_matrix& other) const;
+    bool operator!=(const multi_type_matrix& other) const;
 
-    multi_type_matrix& operator= (const multi_type_matrix& r);
+    multi_type_matrix& operator=(const multi_type_matrix& r);
 
     /**
      * Get a mutable reference of an element (position object) at specified
@@ -551,7 +565,7 @@ public:
      *
      * @param row row index of the element.
      * @param col column index of the element.
-     * @param val new value to set.
+     * @param str new value to set.
      */
     void set(size_type row, size_type col, const string_type& str);
 
@@ -559,7 +573,7 @@ public:
      * Set a string value to an element at specified position.
      *
      * @param pos position of the element to update.
-     * @param val new value to set.
+     * @param str new value to set.
      *
      * @return position of the element block where the new value has been set.
      */
@@ -733,8 +747,8 @@ public:
      *
      * @return function object passed to this method.
      */
-    template<typename _Func>
-    _Func walk(_Func func) const;
+    template<typename FuncT>
+    FuncT walk(FuncT func) const;
 
     /**
      * Walk through the element blocks in a sub-matrix range defined by start
@@ -752,8 +766,8 @@ public:
      *
      * @return function object passed to this method.
      */
-    template<typename _Func>
-    _Func walk(_Func func, const size_pair_type& start, const size_pair_type& end) const;
+    template<typename FuncT>
+    FuncT walk(FuncT func, const size_pair_type& start, const size_pair_type& end) const;
 
     /**
      * Walk through all element blocks in parallel with another matrix
@@ -765,8 +779,8 @@ public:
      *
      * @param right another matrix instance to parallel-walk with.
      */
-    template<typename _Func>
-    _Func walk(_Func func, const multi_type_matrix& right) const;
+    template<typename FuncT>
+    FuncT walk(FuncT func, const multi_type_matrix& right) const;
 
     /**
      * Walk through the element blocks in a sub-matrix range in parallel with
@@ -786,10 +800,9 @@ public:
      *          sub-matrix.  Both column and row must be greater or equal to
      *          those of the start position.
      */
-    template<typename _Func>
-    _Func walk(_Func func, const multi_type_matrix& right,
-        const size_pair_type& start, const size_pair_type& end) const;
-
+    template<typename FuncT>
+    FuncT walk(
+        FuncT func, const multi_type_matrix& right, const size_pair_type& start, const size_pair_type& end) const;
 
 #ifdef MDDS_MULTI_TYPE_MATRIX_DEBUG
     void dump() const
@@ -799,7 +812,6 @@ public:
 #endif
 
 private:
-
     /**
      * Get an array position of the data referenced by the row and column
      * indices.  The array consists of multiple columns, the content of column
@@ -825,7 +837,7 @@ private:
     size_pair_type m_size;
 };
 
-}
+} // namespace mdds
 
 #include "multi_type_matrix_def.inl"
 
